@@ -2,13 +2,14 @@ import re
 import typing  # pylint: disable=unused-import
 
 import astroid  # pylint: disable=unused-import
-import shopify_python.ast
-import six
 
 from pylint import checkers
 from pylint import interfaces
 from pylint import lint  # pylint: disable=unused-import
-from pylint import utils
+
+import six
+
+import shopify_python.ast
 
 
 def register_checkers(linter):  # type: (lint.PyLinter) -> None
@@ -87,6 +88,9 @@ class GoogleStyleGuideChecker(checkers.BaseChecker):
                   'lambda-func',
                   "For common operations like multiplication, use the functions from the operator module"
                   "instead of lambda functions. For example, prefer operator.mul to lambda x, y: x * y."),
+        'C6015': ('No blank line after a class definition',
+                  'blank-line-after-class-required',
+                  'Missing a blank line after a class definition'),
     }
 
     options = (
@@ -121,20 +125,35 @@ class GoogleStyleGuideChecker(checkers.BaseChecker):
 
     BINARY_OPERATORS = {
         "+": "add",
-        "-": "sub",
-        "*": "mul",
+        "in": "contains",
         "/": "truediv",
+        "//": "floordiv",
+        "&": "and_",
+        "^": "xor",
+        "|": "or_",
         "**": "pow",
+        "is": "is",
+        "is not": "is_not",
+        "<<": "lshift",
         "%": "modulo",
+        "*": "mul",
+        ">>": "rshift",
+        "-": "sub",
         "<": "lt",
         "<=": "le",
         "==": "eq",
         "!=": "ne",
         ">=": "ge",
-        ">": "gt",
-        "is": "is"
+        ">": "gt"
     }
 
+    def __init__(self, linter):
+        super(GoogleStyleGuideChecker, self).__init__(linter)
+        name_checker = checkers.base.NameChecker(linter)
+        (regexps, _) = name_checker._create_naming_rules()  # pylint: disable=protected-access
+        self.__class_regexp = regexps['class']
+        self.__const_regexp = regexps['const']
+
     def visit_assign(self, node):  # type: (astroid.Assign) -> None
         self.__avoid_global_variables(node)
 
@@ -165,6 +184,9 @@ class GoogleStyleGuideChecker(checkers.BaseChecker):
     def visit_if(self, node):
         self.__use_cond_expr(node)  # type: (astroid.If) -> None
 
+    def visit_classdef(self, node):  # type: (astroid.ClassDef) -> None
+        self.__class_def_check(node)
+
     @staticmethod
     def __get_module_names(node):  # type: (astroid.ImportFrom) -> typing.Generator[str, None, None]
         for name in node.names:
@@ -207,11 +229,11 @@ class GoogleStyleGuideChecker(checkers.BaseChecker):
         """Avoid global variables."""
 
         def check_assignment(node):
-            if utils.get_global_option(self, 'class-rgx').match(node.name):
+
+            if self.__class_regexp.match(node.name):
                 return  # Type definitions are allowed if they assign to a class name
 
-            if utils.get_global_option(self, 'const-rgx').match(node.name) or \
-               re.match('^__[a-z]+__$', node.name):
+            if self.__const_regexp.match(node.name) or re.match('^__[a-z]+__$', node.name):
                 return  # Constants are allowed
 
             self.add_message('global-variable', node=node, args={'name': node.name})
@@ -287,9 +309,9 @@ class GoogleStyleGuideChecker(checkers.BaseChecker):
         """Prefer Operator Function to Lambda Functions"""
 
         if isinstance(node.body, astroid.UnaryOp):
-            operator = self.UNARY_OPERATORS[node.body.op]
+            operator = self.UNARY_OPERATORS.get(node.body.op)
             argname = node.args.args[0].name
-            if operator and not isinstance(node.body.operand, astroid.BinOp) and argname is node.body.operand.name:
+            if operator and hasattr(node.body.operand, 'name') and argname == node.body.operand.name:
                 varname = node.body.operand.name
                 lambda_fun = "lambda " + varname + ": " + node.body.op + " " + varname
                 op_fun = "operator." + operator
@@ -297,7 +319,7 @@ class GoogleStyleGuideChecker(checkers.BaseChecker):
         elif isinstance(node.body, astroid.BinOp):
             if shopify_python.ast.count_tree_size(node.body) == 3 and len(node.args.args) == 2:
                 node = node.body
-                operator = self.BINARY_OPERATORS[node.op]
+                operator = self.BINARY_OPERATORS.get(node.op)
                 if operator:
                     left = str(node.left.value) if node.left.name == 'int' else node.left.name
                     right = str(node.right.value) if node.right.name == 'int' else node.right.name
@@ -307,10 +329,24 @@ class GoogleStyleGuideChecker(checkers.BaseChecker):
         elif isinstance(node.body, astroid.Compare):
             if shopify_python.ast.count_tree_size(node.body) == 3 and len(node.args.args) == 2:
                 node = node.body
-                operator = self.BINARY_OPERATORS[node.ops[0][0]]
+                operator = self.BINARY_OPERATORS.get(node.ops[0][0])
                 if operator:
                     left = str(node.left.value) if node.left.name == 'int' else node.left.name
                     right = node.ops[0][1].name
                     lambda_fun = "lambda " + left + ', ' + right + ": " + " ".join([left, node.ops[0][0], right])
                     op_fun = "operator." + operator
                     self.add_message('lambda-func', node=node, args={'op': op_fun, 'lambda_fun': lambda_fun})
+
+    def __class_def_check(self, node):  # type: (astroid.ClassDef) -> None
+        """Enforce a blank line after a class definition line."""
+        prev_line = node.lineno
+
+        for element in node.body:
+            curr_line = element.lineno
+            blank_lines = curr_line - prev_line - 1
+            if isinstance(element, astroid.FunctionDef) and blank_lines < 1:
+                self.add_message('blank-line-after-class-required', node=node)
+                break
+            elif isinstance(element, astroid.FunctionDef):
+                break
+            prev_line = curr_line
